@@ -13,9 +13,9 @@ from starlette.applications import Starlette
 from starlette.responses import JSONResponse
 from starlette.routing import Route
 from starlette.exceptions import HTTPException
-from starlette.middleware import Middleware
 from starlette.middleware.base import BaseHTTPMiddleware
 from grpc.experimental.aio import insecure_channel, AioRpcError
+from starlette.middleware.cors import CORSMiddleware
 import os
 from google.protobuf import symbol_database
 from hypercorn.config import Config
@@ -107,53 +107,57 @@ def get_http_handlers(descriptor: FileDescriptor, service_name: str, method: str
     except AioRpcError as ex:
       detail = ex.details()
       grpc_status = ex.code()
-      http_status_code = status_map[StatusCode[grpc_status]]
+      http_status_code = status_map[grpc_status]
       raise HTTPException(http_status_code, detail=detail)
     except Exception as ex:
-      if hasattr(ex, '_state'):
-        detail = ex._state.details
-        grpc_status = ex._state.code.name
-        http_status_code = status_map[StatusCode[grpc_status]]
-      else:
-        detail = str(ex).replace('\n', ' ')
-        http_status_code = 500
+      detail = str(ex).replace('\n', ' ')
+      http_status_code = 500
       raise HTTPException(http_status_code, detail=detail)
 
   return method_get_handler, request_handler
 
 
 async def start_gateway(descriptor: FileDescriptor, port: int = 7070):
-  logging.info('starting gateway...')
-  debug = 'DELPHAI_ENVIRONMENT' in os.environ and os.environ['DELPHAI_ENVIRONMENT'] == 'development'
-  app = Starlette(debug=debug, exception_handlers={HTTPException: http_exception})
-  app.add_middleware(AccessLogMiddleware)
-  app.add_middleware(PrometheusMiddleware)
-  app.add_exception_handler(HTTPException, http_exception)
-  app.add_route("/metrics/", metrics)
-  channel = insecure_channel('localhost:8080')
-  config = Config()
-  config.bind = [f'0.0.0.0:{port}']
-  for service_name in descriptor.services_by_name.keys():
-    service_handler: ServiceDescriptor = descriptor.services_by_name[service_name]
-    if service_handler.full_name.startswith('grpc.'):
-      logging.info(f'skipping service {service_handler.name}')
-    else:
-      logging.info(f'processing service {service_handler.name}')
-      for key in service_handler.methods_by_name.keys():
-        method_get_handler, request_handler = get_http_handlers(descriptor, service_handler.name, key, channel)
-        service_name = key[1:].split('/')[0].split('.')[-1]
-        app.add_route(f'/{key}', route=method_get_handler, methods=['get'])
-        app.add_route(f'/{key}', route=request_handler, methods=['post'])
-        logging.info(f'  processing {key}')
-        method_descriptor: MethodDescriptor = service_handler.methods_by_name[key]
-        method_options: MethodOptions = method_descriptor.GetOptions()
-        fields: List(FieldDescriptor) = method_options.ListFields()
-        for field in fields:
-          if field[0].full_name == 'google.api.http':
-            http_rule: HttpRule = field[1]
-            for supported_method in supported_methods:
-              http_path = getattr(http_rule, supported_method)
-              if http_path != '':
-                app.add_route(http_path, route=request_handler, methods=[supported_method])
-  logging.info(f'started gateway on port {port}')
-  return await serve(app, config)
+  try:
+    logging.info('starting gateway...')
+    debug = 'DELPHAI_ENVIRONMENT' in os.environ and os.environ['DELPHAI_ENVIRONMENT'] == 'development'
+    app = Starlette(debug=debug)
+    app.add_exception_handler(HTTPException, http_exception)
+    app.add_middleware(AccessLogMiddleware)
+    app.add_middleware(PrometheusMiddleware)
+    app.add_middleware(CORSMiddleware,
+                       allow_origins=['*'],
+                       allow_methods=['*'],
+                       allow_headers=['*'],
+                       allow_credentials=True)
+    app.add_exception_handler(HTTPException, http_exception)
+    app.add_route("/metrics/", metrics)
+    channel = insecure_channel('localhost:8080')
+    config = Config()
+    config.bind = [f'0.0.0.0:{port}']
+    for service_name in descriptor.services_by_name.keys():
+      service_handler: ServiceDescriptor = descriptor.services_by_name[service_name]
+      if service_handler.full_name.startswith('grpc.'):
+        logging.info(f'skipping service {service_handler.name}')
+      else:
+        logging.info(f'processing service {service_handler.name}')
+        for key in service_handler.methods_by_name.keys():
+          method_get_handler, request_handler = get_http_handlers(descriptor, service_handler.name, key, channel)
+          service_name = key[1:].split('/')[0].split('.')[-1]
+          app.add_route(f'/{service_handler.full_name}.{key}', route=method_get_handler, methods=['get'])
+          app.add_route(f'/{service_handler.full_name}.{key}', route=request_handler, methods=['post'])
+          logging.info(f'  processing {key}')
+          method_descriptor: MethodDescriptor = service_handler.methods_by_name[key]
+          method_options: MethodOptions = method_descriptor.GetOptions()
+          fields: List(FieldDescriptor) = method_options.ListFields()
+          for field in fields:
+            if field[0].full_name == 'google.api.http':
+              http_rule: HttpRule = field[1]
+              for supported_method in supported_methods:
+                http_path = getattr(http_rule, supported_method)
+                if http_path != '':
+                  app.add_route(http_path, route=request_handler, methods=[supported_method])
+    logging.info(f'started gateway on port {port}')
+    return await serve(app, config)
+  except Exception as ex:
+    logging.error(str(ex))
